import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:knt_bloc/knt_bloc.dart';
import 'package:photo_view/photo_view.dart';
import 'package:photo_view/photo_view_gallery.dart';

import '../../bloc/paging_gallery_bloc.dart';
import '../../config.dart';
import '../../defined.dart';
import '../../model/models.dart';
import '../image/knt_image.dart';
import '../knt_shimmer.dart';

const _defaultDurationInMillis = Duration(milliseconds: 400);

class KntGalleryView extends StatefulWidget {
  const KntGalleryView({
    super.key,
    this.photos = const [],
    this.initIndex,
    this.backgroundColor,
    this.closeButton,
    this.scrollOffsetToLoadMore,
  });

  final List<Photo> photos;
  final int? initIndex;
  final Color? backgroundColor;
  final Widget? closeButton;
  final double? scrollOffsetToLoadMore;

  @override
  State<KntGalleryView> createState() => _KntGalleryViewState();
}

class _KntGalleryViewState extends State<KntGalleryView> {
  late final ValueNotifier<int> _onPhotoChanged;
  late PageController _pageController;
  late List<Photo> _photos;
  late int _currentIndex;

  @override
  void initState() {
    super.initState();
    _photos = widget.photos;
    _currentIndex = widget.initIndex ?? 0;
    _pageController = PageController(initialPage: _currentIndex);
    _onPhotoChanged = ValueNotifier<int>(_currentIndex);
  }

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (_) => PagingGalleryBloc(),
      child: ColoredBox(
        color: Colors.black,
        child: Stack(
          children: [
            PhotoViewGallery.builder(
              pageController: _pageController,
              scrollPhysics: const BouncingScrollPhysics(),
              backgroundDecoration: BoxDecoration(
                color: widget.backgroundColor ?? Colors.black,
              ),
              onPageChanged: _onPageChanged,
              itemCount: _photos.length,
              builder: (context, index) {
                final photo = _photos[index];
                return PhotoViewGalleryPageOptions.customChild(
                  child: _ItemPhoto(
                    photo: photo,
                    width: 300.w,
                    height: 300.w,
                  ),
                  childSize: Size(300.w, 300.w),
                  initialScale: PhotoViewComputedScale.contained,
                  minScale: PhotoViewComputedScale.contained * 0.5,
                  maxScale: PhotoViewComputedScale.covered * 4.1,
                  heroAttributes: PhotoViewHeroAttributes(tag: photo.id),
                );
              },
            ),
            Align(
              alignment: Alignment.bottomCenter,
              child: Container(
                height: 36.w + 16.w * 2,
                padding: EdgeInsets.symmetric(vertical: 16.w),
                decoration: BoxDecoration(
                  gradient: LinearGradient(
                    colors: [
                      Colors.black54,
                      KntUi.shared.secondaryColor.withAlpha(25),
                    ],
                    begin: Alignment.bottomCenter,
                    end: Alignment.topCenter,
                  ),
                ),
                alignment: Alignment.center,
                child: _BottomPhotos(
                  photos: _photos,
                  onPhotoChanged: _onPhotoChanged,
                  scrollOffsetToLoadMore: widget.scrollOffsetToLoadMore,
                  onPickedPhoto: _onPickedPhoto,
                ),
              ),
            ),
            Positioned(
              top: MediaQuery.of(context).padding.top + 12.w,
              right: 12.w,
              child: InkWell(
                onTap: () => _onDismiss(context),
                child: widget.closeButton ??
                    SizedBox(
                      width: 24.w + 20.w,
                      height: 36.w,
                      child: Icon(
                        Icons.close,
                        color: Colors.white,
                        size: 24.w,
                      ),
                    ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  void _onDismiss(BuildContext context) => Navigator.of(context).pop();

  void _onPageChanged(int index) {
    _onPhotoChanged.value = index;
  }

  void _onPickedPhoto(int index) => _pageController.animateToPage(
        index,
        duration: _defaultDurationInMillis,
        curve: Curves.easeInOut,
      );
}

class _BottomPhotos extends StatefulWidget {
  const _BottomPhotos({
    required this.photos,
    required this.onPhotoChanged,
    this.scrollOffsetToLoadMore,
    this.onPickedPhoto,
  });

  final List<Photo> photos;
  final ValueNotifier<int> onPhotoChanged;
  final double? scrollOffsetToLoadMore;
  final void Function(int)? onPickedPhoto;

  @override
  State<_BottomPhotos> createState() => _BottomPhotosState();
}

class _BottomPhotosState extends State<_BottomPhotos> {
  final ScrollController _scrollController = ScrollController();
  late List<Photo> _photos;
  late bool _isLoading;
  late bool _needJumpToIndex;
  late final double _photoWidth;

  @override
  void initState() {
    super.initState();
    _photos = [];
    _isLoading = false;
    _needJumpToIndex = true;
    _photoWidth = 56.w;
    Future.delayed(Duration.zero, () {
      context
          .read<PagingGalleryBloc>()
          .add(LoadGalleryPhotosEvent(widget.photos));
    });
  }

  @override
  Widget build(BuildContext context) {
    return NotificationListener<ScrollNotification>(
      onNotification: (scrollInfo) {
        if (!_isLoading &&
            scrollInfo is ScrollUpdateNotification &&
            (scrollInfo.metrics.pixels - scrollInfo.metrics.maxScrollExtent >
                (widget.scrollOffsetToLoadMore ?? 20.0))) {
          _onLoadMoreResult();
        }
        return false;
      },
      child: BlocConsumer<PagingGalleryBloc, BaseState>(
        listener: (context, state) {
          if (!mounted) {
            return;
          }
          if (state is LoadedGalleryPhotosState) {
            _onFirstQuery();
          }
          if (state is BasePagingResultState<Photo>) {
            _jumpToCurrentPhoto();
          }
        },
        builder: (context, state) {
          _isLoading = false;
          if (state is BaseLoadingState) {
            _isLoading = true;
          }
          if (state is BasePagingResultState<Photo>) {
            _photos = state.data;
          }
          return ValueListenableBuilder<int>(
            valueListenable: widget.onPhotoChanged,
            builder: (context, currentIndex, child) => ListView.builder(
              controller: _scrollController,
              physics: const BouncingScrollPhysics(),
              scrollDirection: Axis.horizontal,
              padding: EdgeInsets.symmetric(horizontal: 16.w),
              itemBuilder: (context, index) {
                if (index >= _photos.length) {
                  return KntShimmer(
                    width: _photoWidth,
                    height: 36.w,
                  );
                }
                final photo = _photos[index];
                return InkWell(
                  onTap: () => widget.onPickedPhoto?.call(index),
                  child: _ItemPhoto(
                    photo: photo,
                    asCover: false,
                    needBorder: currentIndex == index,
                  ),
                );
              },
              itemCount: _isLoading ? _photos.length + 3 : _photos.length,
            ),
          );
        },
      ),
    );
  }

  void _onFirstQuery() {
    if (!mounted) {
      return;
    }
    context.read<PagingGalleryBloc>().add(
          const BasePagingEvent<String?>(
            limit: DefinedUI.defaultFetchLimitGallery,
            isFirstRequest: true,
          ),
        );
  }

  void _onLoadMoreResult() {
    if (!mounted) {
      return;
    }
    context.read<PagingGalleryBloc>().add(
          const BasePagingEvent<String?>(
            limit: DefinedUI.defaultFetchLimitGallery,
          ),
        );
  }

  void _jumpToCurrentPhoto() {
    if (!_needJumpToIndex) {
      return;
    }
    final currentIndex = widget.onPhotoChanged.value;
    _scrollController.animateTo(
      _photoWidth * currentIndex,
      duration: const Duration(milliseconds: 200),
      curve: Curves.linear,
    );
    _needJumpToIndex = false;
  }
}

class _ItemPhoto extends StatelessWidget {
  const _ItemPhoto({
    required this.photo,
    this.asCover = true,
    this.width,
    this.height,
    this.needBorder = false,
  });

  final Photo photo;
  final bool asCover;
  final double? width;
  final double? height;
  final bool needBorder;

  @override
  Widget build(BuildContext context) {
    final nonNullWidth = width ?? 56.w;
    final nonNullHeight = height ?? 36.w;
    final uri = asCover ? photo.coverUri : photo.thumbnailUri;
    return Container(
      width: nonNullWidth,
      height: nonNullHeight,
      decoration: BoxDecoration(
        border: Border.all(
          width: 3.w,
          color: needBorder
              ? KntUi.shared.primaryColor.withAlpha(124)
              : Colors.transparent,
        ),
      ),
      child: KntImage(
        uri: uri,
        width: nonNullWidth,
        height: nonNullHeight,
        imageType: photo.storageType,
      ),
    );
  }
}
