import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../generated/colors.gen.dart';

class KntSimpleEditText extends StatefulWidget {
  const KntSimpleEditText({
    super.key,
    this.defaultValue,
    this.hintText,
    this.textColor,
    this.hintTextColor,
    this.contentPadding,
    this.fontSize,
    this.fontWeight,
    this.maxLine,
    this.maxCharacters,
    this.onTextChanged,
    this.textInputAction,
    this.autoFocus = false,
  });

  final String? defaultValue;
  final String? hintText;
  final Color? textColor;
  final Color? hintTextColor;
  final EdgeInsets? contentPadding;
  final double? fontSize;
  final FontWeight? fontWeight;
  final int? maxLine;
  final int? maxCharacters;
  final void Function(String)? onTextChanged;
  final TextInputAction? textInputAction;
  final bool autoFocus;

  @override
  State<KntSimpleEditText> createState() => _KntSimpleEditTextState();
}

class _KntSimpleEditTextState extends State<KntSimpleEditText> {
  late TextEditingController _editController;

  @override
  void initState() {
    super.initState();
    _editController = TextEditingController(text: widget.defaultValue);
    Future.delayed(Duration.zero, () {
      FocusScope.of(context).unfocus();
    });
  }

  @override
  void dispose() {
    super.dispose();
    _editController.dispose();
  }

  @override
  void didUpdateWidget(covariant KntSimpleEditText oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.defaultValue != widget.defaultValue) {
      setState(() {
        _editController
          ..text = widget.defaultValue ?? ''
          ..selection = TextSelection.fromPosition(
            TextPosition(offset: _editController.text.length),
          );
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      controller: _editController,
      autofocus: widget.autoFocus,
      maxLines: widget.maxLine,
      maxLength: widget.maxCharacters,
      onChanged: widget.onTextChanged,
      textInputAction: widget.textInputAction,
      style: TextStyle(
        color: widget.textColor ?? ColorName.tundora,
        fontSize: widget.fontSize ?? 14.sp,
        fontWeight: widget.fontWeight ?? FontWeight.w400,
        height: 1.2,
      ),
      decoration: InputDecoration(
        contentPadding: widget.contentPadding,
        hintText: widget.hintText,
        hintStyle: TextStyle(
          color: widget.hintTextColor ?? ColorName.doveGray,
          fontSize: widget.fontSize ?? 14.sp,
          fontWeight: widget.fontWeight ?? FontWeight.w400,
          height: 1.2,
        ),
        border: InputBorder.none,
      ),
    );
  }
}

class KntEditText extends StatefulWidget {
  const KntEditText({
    super.key,
    this.isSecureText = false,
    this.styleSeparate = false,
    this.defaultValue,
    this.hintText,
    this.icon,
    this.contentPadding,
    this.focusNode,
    this.onFieldSubmitted,
    this.textInputAction,
    this.title,
    this.borderColor,
    this.focusBorderColor,
    this.textColor,
    this.hintTextColor,
    this.boxColor,
    this.fontSize,
    this.borderRadius,
    this.borderWidth,
    this.maxLength = 200,
    this.onTextChanged,
  });

  final bool isSecureText;
  final bool styleSeparate;
  final String? defaultValue;
  final String? hintText;
  final Widget? icon;
  final EdgeInsets? contentPadding;
  final FocusNode? focusNode;
  final void Function(String)? onFieldSubmitted;
  final TextInputAction? textInputAction;
  final String? title;
  final Color? borderColor;
  final Color? focusBorderColor;
  final Color? textColor;
  final Color? hintTextColor;
  final Color? boxColor;
  final double? fontSize;
  final BorderRadius? borderRadius;
  final double? borderWidth;
  final int? maxLength;
  final void Function(String)? onTextChanged;

  @override
  State<KntEditText> createState() => _KntEditTextState();
}

class _KntEditTextState extends State<KntEditText> {
  late TextEditingController _editController;

  @override
  void initState() {
    super.initState();
    _editController = TextEditingController(text: widget.defaultValue);
  }

  @override
  void dispose() {
    super.dispose();
    _editController.dispose();
  }

  @override
  void didUpdateWidget(covariant KntEditText oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.defaultValue != widget.defaultValue) {
      setState(() {
        _editController.text = widget.defaultValue ?? '';
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    final styleSeparate = widget.styleSeparate;
    final noneEmptyTitle = widget.title?.isNotEmpty ?? false;
    final borderRadius = widget.borderRadius ?? BorderRadius.circular(8.w);
    final textField = TextFormField(
      controller: _editController,
      onChanged: widget.onTextChanged,
      textInputAction: widget.textInputAction,
      obscureText: widget.isSecureText,
      onFieldSubmitted: widget.onFieldSubmitted,
      focusNode: widget.focusNode,
      minLines: styleSeparate ? 3 : 1,
      maxLines: styleSeparate ? 10 : 1,
      maxLength: widget.maxLength,
      style: TextStyle(
        color: widget.textColor ?? ColorName.doveGray,
        fontSize: widget.fontSize,
      ),
      decoration: InputDecoration(
        contentPadding: widget.contentPadding,
        focusedBorder: styleSeparate
            ? OutlineInputBorder(
                borderRadius: borderRadius,
                borderSide: BorderSide.none,
              )
            : UnderlineInputBorder(
                borderRadius: borderRadius,
                borderSide: BorderSide(
                  color: widget.focusBorderColor ?? ColorName.doveGray,
                  width: widget.borderWidth ?? 1.w,
                ),
              ),
        border: styleSeparate
            ? OutlineInputBorder(
                borderRadius: borderRadius,
                borderSide: BorderSide.none,
              )
            : UnderlineInputBorder(
                borderRadius: borderRadius,
                borderSide: BorderSide(
                  color: widget.borderColor ?? ColorName.doveGray,
                  width: widget.borderWidth ?? 1.w,
                ),
              ),
        hintText: widget.hintText ?? '',
        hintStyle: TextStyle(
          color: widget.hintTextColor ?? ColorName.doveGray,
          fontSize: widget.fontSize ?? 14.sp,
        ),
      ),
    );
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        if (styleSeparate && noneEmptyTitle)
          Text(
            widget.title ?? '',
            style: TextStyle(
              color: widget.textColor ?? ColorName.tundora,
              fontSize: widget.fontSize ?? 14.sp,
            ),
          ),
        if (styleSeparate && noneEmptyTitle) SizedBox(height: 12.w),
        if (styleSeparate)
          Container(
            padding: EdgeInsets.only(
              left: 2.w,
              right: 2.w,
              bottom: 12.w,
            ),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(8.w),
              border: Border.all(
                color: widget.borderColor ?? ColorName.tundora,
                width: widget.borderWidth ?? 1.w,
              ),
              color: widget.boxColor ?? ColorName.tundora,
            ),
            child: textField,
          )
        else
          textField,
      ],
    );
  }
}
